import UIKit
import WebKit
import Flutter

public class DpayDelegate: UIViewController, WKUIDelegate, WKScriptMessageHandler, WKNavigationDelegate {

    var webView: WKWebView!
    var options: NSMutableDictionary!
    var checkoutUrl: String = ""
    var pendingResult: FlutterResult!
    var childWebView: WKWebView!
    let snowplowManager = SnowplowManager.shared
    let sessionID = UUID().uuidString
    let backButton = UIButton()
    let navBar = UINavigationBar()
    let navBarHeight = CGFloat(50)
    var navBarColor = UIColor.white
    let bundle = Bundle(for: DpayDelegate.self)
    var backImage = UIImage()
    var activityIndicatorWidth = CGFloat(80)
    var activityIndicatorHeight = CGFloat(80)
    var activityIndicator = UIActivityIndicatorView(style: .gray)
    
    public override func loadView() {
        let wkpreferences = WKPreferences()
        wkpreferences.javaScriptCanOpenWindowsAutomatically = true
        wkpreferences.javaScriptEnabled = true
        let webConfiguration = WKWebViewConfiguration()
        webConfiguration.preferences = wkpreferences
        let userController: WKUserContentController = WKUserContentController()
        webConfiguration.userContentController = userController
        webConfiguration.userContentController.add(self, name: "dpayIOS")
        webView = WKWebView(frame: .zero, configuration: webConfiguration)
        webView.uiDelegate = self
        webView.navigationDelegate = self
        view = webView
     }
    
    public override func viewDidLoad() {
        super.viewDidLoad()
        
        let midX = (UIScreen.main.bounds.size.width - activityIndicatorWidth)/2
        let midY = (UIScreen.main.bounds.size.height - activityIndicatorHeight)/2
        activityIndicator.color = UIColor.red
        activityIndicator.center = self.view.center
        activityIndicator.transform = CGAffineTransform(scaleX: 2, y: 2)
        activityIndicator.frame = CGRect(x: midX, y: midY, width: activityIndicatorWidth, height: activityIndicatorHeight)
        view.addSubview(activityIndicator)
        
        if let resourceBundleURL = bundle.url(forResource: "DpaySDK", withExtension: "bundle"),
           let resourceBundle = Bundle(url: resourceBundleURL){
            if let image = UIImage.init(named: "back-arrow", in: resourceBundle, compatibleWith: nil) {
                backImage = image
            }
        } else {
                    fatalError("DpaySDK bundle not found in \(bundle)")
                }
        
        do {
            self.navigationItem.hidesBackButton = true
            self.navigationController?.isNavigationBarHidden = true
            if let url = try openCheckout(checkoutoptions: options).addingPercentEncoding(withAllowedCharacters: .urlQueryAllowed) {
                if let myURL = URL(string: url) {
                    print(myURL)
                    let myRequest = URLRequest(url: myURL)
                    webView.load(myRequest)
                }
            }
        } catch let error as NSError {
            NSLog("error: " + String(describing: error.description))
        }
     }
    
    public func userContentController(_ userContentController: WKUserContentController, didReceive message: WKScriptMessage) {
        NSLog("message\n" + String(describing: message.body))
        pendingResult(String(describing: message.body) as String)
        self.dismiss(animated: true, completion: nil)
        let jsonString = String(describing: message.body)
        let jsonData = jsonString.data(using: .utf8)!
        let json: AnyObject = try! JSONSerialization.jsonObject(with: jsonData, options: .allowFragments) as AnyObject
        let response: AnyObject = json["response"] as AnyObject
        
        snowplowManager?.setTracker()
        if let order_id = response["order_id"] as? String,
           let currency = response["currency"] as? String,
           let payment_method_used = response["payment_method_used"] as? String,
           let amountString = response["amount"] as? String,
           let transaction_type = json["type"] as? String,
           let amount = Double(amountString) {
            NSLog("trackCompletion: " + sessionID)
            snowplowManager?.trackCompletion(
                customerId: String(describing: options.object(forKey: "customerId") as! NSObject),
                orderId: order_id,
                currency: currency,
                event_name: "payment_" + transaction_type,
                payment_method_used: payment_method_used,
                count: 1,
                amount: amount,
                sessionId: sessionID)
        }
        webView.stopLoading()
        self.webView = nil
    }
    
    public func webView(_ webView: WKWebView, createWebViewWith configuration:WKWebViewConfiguration, for navigationAction: WKNavigationAction, windowFeatures: WKWindowFeatures) -> WKWebView? {
        if #available(iOS 11.0, *) {
            childWebView = WKWebView(frame: CGRect(x: 0, y: (view.safeAreaInsets.top + navBarHeight), width: view.frame.size.width, height: view.frame.size.height - navBarHeight - view.safeAreaInsets.top), configuration: configuration)
        } else {
            childWebView = WKWebView(frame: CGRect(x: 0, y: (topLayoutGuide.length + navBarHeight), width: view.frame.size.width, height: (view.frame.size.height - navBarHeight - topLayoutGuide.length)), configuration: configuration)
        }
        childWebView.autoresizingMask = [.flexibleWidth, .flexibleHeight]
        childWebView.navigationDelegate = self
        childWebView.uiDelegate = self
        let url = navigationAction.request.url!
        childWebView.load(URLRequest(url: url))
        view.addSubview(childWebView)

        if #available(iOS 11.0, *) {
            navBar.frame = CGRect(x: 0, y: view.safeAreaInsets.top, width: view.frame.size.width, height: navBarHeight)
        } else {
            navBar.frame = CGRect(x: 0, y: topLayoutGuide.length, width: view.frame.size.width, height: navBarHeight)
        }
        navBar.backgroundColor = navBarColor
        view.addSubview(navBar)
        
        //set the position of the button
        if #available(iOS 11.0, *) {
            backButton.frame = CGRect(x: 0, y: view.safeAreaInsets.top, width: 50, height: navBarHeight)
        } else {
            backButton.frame = CGRect(x: 0, y: topLayoutGuide.length, width: 50, height: navBarHeight)
        }
        
        backButton.setImage(backImage, for: .normal)
        backButton.contentVerticalAlignment = .center
        backButton.contentHorizontalAlignment = .left
        backButton.imageEdgeInsets = UIEdgeInsets(top: 0, left: 15, bottom: 0, right: 0)
        backButton.addTarget(self, action: #selector(buttonAction), for: .touchUpInside)
        view.addSubview(backButton)
        
        return childWebView
    }
    
    @objc
      func buttonAction() {
        print("Button pressed")
        if (childWebView.canGoBack) {
            childWebView.goBack()
        } else {
            childWebView.removeFromSuperview()
            childWebView = nil
            backButton.removeFromSuperview()
            navBar.removeFromSuperview()
        }
      }
    
    public func webView(_ webView: WKWebView, decidePolicyFor navigationAction: WKNavigationAction, decisionHandler: @escaping (WKNavigationActionPolicy) -> Void) {
        
        let request = navigationAction.request
        let requestURL = request.url?.absoluteString
           
        var red: CGFloat = 0
        var green: CGFloat = 0
        var blue: CGFloat = 0
        var alpha: CGFloat = 0
        
        if (requestURL?.contains(Constants.DANA_URL_IDENTIFIER) == true) {navBar.backgroundColor = Constants.DANA_COLOR}
        if (requestURL?.contains(Constants.LINKAJA_URL_IDENTIFIER) == true) {navBar.backgroundColor = Constants.LINKAJA_COLOR}
        if (requestURL?.contains(Constants.SHOPEEPAY_URL_IDENTIFIER) == true) {navBar.backgroundColor = Constants.SHOPEEPAY_COLOR}
        if (requestURL?.contains(Constants.MANDIRI_URL_IDENTIFIER) == true) {navBar.backgroundColor = Constants.MANDIRI_COLOR}
        if (requestURL?.contains(Constants.BNI_URL_IDENTIFIER) == true) {navBar.backgroundColor = Constants.BNI_COLOR}
        if (requestURL?.contains(Constants.BRI_URL_IDENTIFIER) == true) {navBar.backgroundColor = Constants.BRI_COLOR}
        if (requestURL?.contains(Constants.PERMATA_URL_IDENTIFIER) == true) {navBar.backgroundColor = Constants.PERMATA_COLOR}
        if (requestURL?.contains(Constants.BCA_URL_IDENTIFIER) == true) {navBar.backgroundColor = Constants.BCA_COLOR}
        if (requestURL?.contains(Constants.STATIC_LOADING_URL_IDENTIFIER) == true) {navBar.backgroundColor = Constants.STATIC_COLOR}
        
        navBar.backgroundColor?.getRed(&red, green: &green, blue: &blue, alpha: &alpha)
        if (red*0.299*255 + green*0.587*255 + blue*0.114*255) < 150 {
            if let resourceBundleURL = bundle.url(forResource: "DpaySDK", withExtension: "bundle"),
               let resourceBundle = Bundle(url: resourceBundleURL){
                if let image = UIImage.init(named: "back-arrow-light", in: resourceBundle, compatibleWith: nil) {
                    backImage = image
                }
            } else {
                        fatalError("DpaySDK bundle not found in \(bundle)")
                    }
            backButton.setImage(backImage, for: .normal)
        } else {
            if let resourceBundleURL = bundle.url(forResource: "DpaySDK", withExtension: "bundle"),
               let resourceBundle = Bundle(url: resourceBundleURL){
                if let image = UIImage.init(named: "back-arrow", in: resourceBundle, compatibleWith: nil) {
                    backImage = image
                }
            } else {
                        fatalError("DpaySDK bundle not found in \(bundle)")
                    }
            backButton.setImage(backImage, for: .normal)
        }
        
        if (Bool((request.url?.absoluteString.hasPrefix("app://"))!) == true) {
            if (Bool((requestURL?.contains("ovo"))!)) {
                let separators = CharacterSet(charactersIn: "\\:\\/\\/")
                if let urlParts = request.url?.absoluteString.components(separatedBy: separators) {
                    let packageName = urlParts[urlParts.count-1]
                    let custom_url = packageName + "://"
                    if #available(iOS 10.0, *) {
                        UIApplication.shared.open(URL(string: custom_url)!)
                    } else {
                        UIApplication.shared.openURL(URL(string: custom_url)!)
                    }

                }
            }
        }
        decisionHandler(WKNavigationActionPolicy.allow)
    }
    
    public func webView(_ webView: WKWebView, decidePolicyFor navigationResponse: WKNavigationResponse, decisionHandler: @escaping (WKNavigationResponsePolicy) -> Void) {
        
        if let response = navigationResponse.response as? HTTPURLResponse {
            if (response.statusCode == 401 || response.statusCode == 500) {
                NSLog("error with code: " + String(describing: response.statusCode))
                pendingResult(String(describing: String(describing: response.description)) as String)
                self.dismiss(animated: true, completion: nil)
                }
            }
        decisionHandler(.allow)
    }
    
    public func webView(_ webView: WKWebView, didFinish navigation: WKNavigation!) {
        let defaults = UserDefaults.standard
        if let date = defaults.object(forKey: "previousRequest") as? Date {
            let daysPassed = Calendar.current.dateComponents([.day], from: date, to: Date()).day ?? 0
            if daysPassed > 0 {
                defaults.set(Date(), forKey: "previousRequest")
                var installedApps = ""
                let removeItems = "window.localStorage.removeItem('installed_apps');" +
                                "window.localStorage.removeItem('installed_bank_apps');" +
                                "window.localStorage.removeItem('ovo_app');";

                webView.evaluateJavaScript(removeItems, completionHandler: nil)
                if (checkIfInstalled(customURL: Constants.OVO_CUSTOM_URL)) {
                    webView.evaluateJavaScript("window.localStorage.setItem('ovo_app','ovo');", completionHandler: nil)
                }

                for app in Constants.APPWATCHLIST {
                    if (checkIfInstalled(customURL: app)) {
                        installedApps.append(Constants.BANKVAPACKAGES[app] ?? "")
                        installedApps.append(",")
                    }
                }
                
                if (!installedApps.isEmpty) {
                    installedApps.removeLast()
                }
                
                webView.evaluateJavaScript("window.localStorage.setItem('installed_apps','" + installedApps + "');", completionHandler: nil)
            }
        } else {
            defaults.set(Date(), forKey: "previousRequest")
        }

        showActivityIndicator(show: false)
    }
    
    public func webView(_ webView: WKWebView, didStartProvisionalNavigation navigation: WKNavigation!) {
        snowplowManager?.setTracker()
        NSLog("trackWebview: " + sessionID)
        snowplowManager?.trackWebview(
            customerId: String(describing: options.object(forKey: "customerId") as! NSObject),
            orderId: String(describing: options.object(forKey: "orderId") as! NSObject),
            currency: String(describing: options.object(forKey: "currency") ?? ""),
            event_name: "webview_opened",
            sessionId: sessionID)
        
        showActivityIndicator(show: true)
    }
    
    public func webView(_ webView: WKWebView, didFailProvisionalNavigation navigation: WKNavigation!, withError error: Error) {
        NSLog("error while loading: " + error.localizedDescription)
        showActivityIndicator(show: false)
    }
    
    public func webView(_ webView: WKWebView, didFail navigation: WKNavigation!, withError error: Error) {
        NSLog("failed navigation: error " + error.localizedDescription)
        showActivityIndicator(show: false)
    }
    
    public func webViewDidClose(_ webView: WKWebView) {
        webView.removeFromSuperview()
        childWebView = nil
        backButton.removeFromSuperview()
        navBar.removeFromSuperview()
    }
    
    public func showActivityIndicator(show: Bool) {
        if show {
            activityIndicator.startAnimating()
        }
        else {
            activityIndicator.stopAnimating()
        }
    }
    
    public func openCheckout(checkoutoptions:NSMutableDictionary) throws -> String {
        options = checkoutoptions
        checkoutUrl += Constants.CHECKOUT_URL

        checkoutUrl += try addQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&locale=", options: options, errorTag: "Locale", key: "locale")
        checkoutUrl += try addQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&environment=", options: options, errorTag: "Environment", key: "environment")
        checkoutUrl += try addQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&site_name=", options: options, errorTag: "Site name", key: "siteName")
        checkoutUrl += try addQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&customer_email=", options: options, errorTag: "Customer email", key: "customerEmail")
        checkoutUrl += try addQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&access_key=", options: options, errorTag: "Access token", key: "accessToken")
        checkoutUrl += try addQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&order_id=", options: options, errorTag: "Order id", key: "orderId")
        checkoutUrl += try addQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&customer_id=", options: options, errorTag: "Customer ID", key: "customerId")
        checkoutUrl += "&session_id=" + sessionID
        
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&amount=", options: options, key: "amount")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&currency=", options: options, key: "currency")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&type=", options: options, key: "paymentType")
        
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&theme_color=", options: options, key: "themeColor")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&accent_color=", options: options, key: "accentColor")
        
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&label=", options: options, key: "label")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&receiver_name=", options: options, key: "receiverName")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&receiver_phone=", options: options, key: "receiverPhone")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&landmark=", options: options, key: "landmark")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&customer_given_name=", options: options, key: "customerGivenName")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&customer_mobile=", options: options, key: "customerMobile")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&customer_address_line1=", options: options, key: "customerAddressLine1")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&customer_address_line2=", options: options, key: "customerAddressLine2")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&customer_city=", options: options, key: "customerCity")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&customer_region=", options: options, key: "customerRegion")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&customer_country=", options: options, key: "customerCountry")
        checkoutUrl += try addOptionalQueryParameterToCheckoutUrl(checkoutUrl: checkoutUrl, query: "&customer_postal_code=", options: options, key: "customerPostalCode")
        
        
        return checkoutUrl
    }
}
