import Foundation
@_implementationOnly import SnowplowTracker

class SnowplowManager: NSObject {
    @objc var tracker: SPTracker?
    static var shared: SnowplowManager? = SnowplowManager.init()
    
    func setTracker() {
        let emitter = SPEmitter.build({ builder in
            builder?.setUrlEndpoint(Constants.SNOWPLOW_URL_PRODUCTION_ENDPOINT) //set the URL where you send the data
            builder?.setProtocol(.https) // protocol i.e., either http or https
        })
        
        tracker = SPTracker.build({ builder in
            builder?.setEmitter(emitter) // set the emitter of the tracker
            builder?.setAppId(Constants.SNOWPLOW_APP_ID) // track the appId
    
            let subject = SPSubject.init() // set a subject
            builder?.setSubject(subject) // add subject to the tracker
        })
    }
    
    func trackWebview(customerId: String, orderId: String, currency: String, event_name: String, sessionId: String) {
        let clicksSchema = Constants.CLICKS_SCHEMA
         let data = [
            "param1": "webview_iOS",
            Constants.CUSTOMER_ID_TAG : customerId,
            Constants.ORDER_ID_TAG : orderId,
            Constants.MERCHANT_ID_TAG : "",
            Constants.PAYMENT_AMOUNT_TAG : 0,
            Constants.CURRENCY_TAG : currency,
            Constants.EVENT_NAME_TAG : event_name,
            Constants.SESSION_ID_TAG : sessionId
         ] as [String : Any]
         let clicksContext = SPSelfDescribingJson(schema: clicksSchema, andData: data as NSObject?)

        tracker?.trackSelfDescribingEvent(clicksContext)
    }
    
    func trackCompletion(customerId: String,
                         orderId: String,
                         currency: String,
                         event_name: String,
                         payment_method_used: String,
                         count: Int,
                         amount: Double,
                         sessionId: String) {
        let countSchema = Constants.COUNT_SCHEMA
         let data = [
            "param1": payment_method_used,
            Constants.CUSTOMER_ID_TAG : customerId,
            Constants.ORDER_ID_TAG : orderId,
            Constants.MERCHANT_ID_TAG : "",
            Constants.PAYMENT_AMOUNT_TAG : round(amount),
            Constants.CURRENCY_TAG : currency,
            Constants.EVENT_COUNT_TAG: count,
            Constants.EVENT_NAME_TAG : event_name,
            Constants.SESSION_ID_TAG : sessionId
         ] as [String : Any]
         let countContext = SPSelfDescribingJson(schema: countSchema, andData: data as NSObject?)

        tracker?.trackSelfDescribingEvent(countContext)
    }
}
